from flask_restful import Resource, request, abort
from marshmallow import Schema, fields
import json
import os
import random
import string
from models.client_model import ClientModel


class ClientSchema(Schema):
    client_id = fields.Str(required=True)
    client_name = fields.Str(required=True)
    subscription = fields.Str(required=True)


# Client
class Client(Resource):
    clients_file = None

    def __init__(self):
        self.schema = ClientSchema()
        self.clients_file = ClientModel()

    def get(self, client_id):
        client = self.clients_file.find(client_id)
        del client["secret"]
        if not client:
            abort(404, message="No client with id: {}".format(client_id))
        return client

    def delete(self, client_id):
        client = self.clients_file.find(client_id)
        if not client:
            abort(404, message="No client with id: {}".format(client_id))
        else:
            self.clients_file.remove(client)
        return client, 204

    def put(self, client_id):
        args = request.get_json()
        errors = self.schema.validate(args)
        if errors:
            abort(400, message=str(errors))

        client = self.clients_file.find(client_id)
        if not client:
            abort(404, message="No client with id: {}".format(client_id))
        else:
            client = self.clients_file.update(args)
        return client, 201


# ClientList
class ClientList(Resource):
    def __init__(self):
        self.schema = ClientSchema()
        self.clients_file = ClientModel()

    def get(self):
        return {
            client_id: (client, client.pop("secret", None))[0]
            for client_id, client in self.clients_file.get().items()
        }

    def post(self):
        args = request.get_json()

        errors = self.schema.validate(args)
        if errors:
            abort(400, message=str(errors))

        client = self.clients_file.find(args["client_id"])
        if client:
            abort(403, message="Duplicate client id: {}".format(client_id))
        else:
            client = self.clients_file.add(args)
        return client, 201
